@@ -42,8 +42,8 @@ from typing import Iterable, List, Tuple, Union
 import numpy as np
 import scipy.optimize as sopt
 import torch
-import torch.nn as nn
 from sklearn.base import BaseEstimator
+from torch import nn
 
 from causalnex.structure.pytorch.dist_type._base import DistTypeBase
 from causalnex.structure.pytorch.nonlinear import LocallyConnected
@@ -64,6 +64,7 @@ class NotearsMLP(nn.Module, BaseEstimator):
         n_features: int,
         dist_types: List[DistTypeBase],
         use_bias: bool = False,
+        use_gpu: bool = True,
         hidden_layer_units: Iterable[int] = (0,),
         bounds: List[Tuple[int, int]] = None,
         lasso_beta: float = 0.0,
@@ -77,6 +78,7 @@ class NotearsMLP(nn.Module, BaseEstimator):
             n_features: number of input features.
             dist_types: list of data type objects used to fit the NOTEARS algorithm.
             use_bias: True to add the intercept to the model
+            use_gpu: use gpu if it is set to True and CUDA is available
             hidden_layer_units: An iterable where its length determine the number of layers used,
             and the numbers determine the number of nodes used for the layer in order.
             bounds: bound constraint for each parameter.
@@ -88,7 +90,15 @@ class NotearsMLP(nn.Module, BaseEstimator):
             Prevents the weights from being scaled above 1/nonlinear_clamp.
         """
         super().__init__()
-        self.device = torch.device("cpu")
+
+        avail_device = "cuda" if torch.cuda.is_available() and use_gpu else "cpu"
+        self.device = torch.device(avail_device)
+
+        self._logger.info(
+            "PyTorch backend for %s is running on '%s'",
+            self.__class__.__name__,
+            avail_device.upper(),
+        )
         self.lasso_beta = lasso_beta
         self.ridge_beta = ridge_beta
         self.nonlinear_clamp = nonlinear_clamp
@@ -99,12 +109,14 @@ class NotearsMLP(nn.Module, BaseEstimator):
             if hidden_layer_units[0]
             else [n_features, 1]
         )
-
         # dag_layer: initial linear layer
         self.dag_layer = nn.Linear(
             self.dims[0], self.dims[0] * self.dims[1], bias=use_bias
         ).float()
+
+        self.dag_layer.to(self.device)  # Send DAG layer to device
         nn.init.zeros_(self.dag_layer.weight)
+
         if use_bias:
             nn.init.zeros_(self.dag_layer.bias)
 
@@ -116,8 +128,10 @@ class NotearsMLP(nn.Module, BaseEstimator):
             for input_features, output_features in zip(self.dims[1:-1], self.dims[2:])
         ]
         self._loc_lin_layer_weights = nn.ModuleList(layers)
+
         for layer in layers:
             layer.reset_parameters()
+            layer.to(self.device)
 
         # set the bounds as an attribute on the weights object
         self.dag_layer.weight.bounds = bounds
@@ -135,6 +149,7 @@ class NotearsMLP(nn.Module, BaseEstimator):
     def dag_layer_bias(self) -> Union[torch.Tensor, None]:
         """
         dag_layer bias is the bias of the first fully connected layer which determines the causal structure.
+
         Returns:
             dag_layer bias if use_bias is True, otherwise None
         """
@@ -144,6 +159,7 @@ class NotearsMLP(nn.Module, BaseEstimator):
     def dag_layer_weight(self) -> torch.Tensor:
         """
         dag_layer weight is the weight of the first fully connected layer which determines the causal structure.
+
         Returns:
             dag_layer weight
         """
@@ -153,6 +169,7 @@ class NotearsMLP(nn.Module, BaseEstimator):
     def loc_lin_layer_weights(self) -> torch.Tensor:
         """
         loc_lin_layer weights are the weight of hidden layers after the first fully connected layer.
+
         Returns:
             loc_lin_layer weights
         """
@@ -170,16 +187,16 @@ class NotearsMLP(nn.Module, BaseEstimator):
         """
         x = self.dag_layer(x)  # [n, d * m1]
         x = x.view(-1, self.dims[0], self.dims[1])  # [n, d, m1]
+
         for output_dim, layer in zip(self.dims[1:], self.loc_lin_layer_weights):
             x = torch.sigmoid(x)  # [n, d, m1]
-
             x = nn.LayerNorm(
                 output_dim,
                 eps=self.nonlinear_clamp,
                 elementwise_affine=True,
             )(x)
-
             x = layer(x)  # [n, d, m2]
+
         x = x.squeeze(dim=2)  # [n, d]
         return x
 
@@ -232,6 +249,7 @@ class NotearsMLP(nn.Module, BaseEstimator):
     ):
         """
         Fit NOTEARS MLP model using the input data x
+
         Args:
             x: 2d numpy array input data, axis=0 is data rows, axis=1 is data columns. Data must be row oriented.
             max_iter: max number of dual ascent steps during optimisation.
@@ -243,6 +261,7 @@ class NotearsMLP(nn.Module, BaseEstimator):
 
         for n_iter in range(max_iter):
             rho, alpha, h = self._dual_ascent_step(X_torch, rho, alpha, h, rho_max)
+
             if h <= h_tol or rho >= rho_max:
                 break
             if n_iter == max_iter - 1 and h > h_tol:
@@ -308,12 +327,14 @@ class NotearsMLP(nn.Module, BaseEstimator):
                 flatten vector of bound constraints for each parameter in numpy form
             """
             bounds = []
+
             for p in params:
                 try:
                     b = p.bounds
                 except AttributeError:
                     b = [(None, None)] * p.numel()
                 bounds += b
+
             return bounds
 
         def _get_flat_params(params: List[torch.Tensor]) -> np.ndarray:
@@ -343,8 +364,10 @@ class NotearsMLP(nn.Module, BaseEstimator):
                 flat_params: parameters in the form of flatten vector
             """
             offset = 0
-            flat_params_torch = torch.from_numpy(flat_params).to(
-                torch.get_default_dtype()
+            flat_params_torch = (
+                torch.from_numpy(flat_params)
+                .to(torch.get_default_dtype())
+                .to(self.device)
             )
             for p in params:
                 n_params = p.numel()
@@ -366,11 +389,10 @@ class NotearsMLP(nn.Module, BaseEstimator):
             optimizer.zero_grad()
 
             n_features = X.shape[1]
-
             X_hat = self(X)
             h_val = self._h_func()
-
             loss = 0.0
+
             # sum the losses across all dist types
             for dist_type in self.dist_types:
                 loss = loss + dist_type.loss(X, X_hat)
@@ -387,13 +409,12 @@ class NotearsMLP(nn.Module, BaseEstimator):
             flat_grad = _get_flat_grad(params)
             return loss, flat_grad.astype("float64")
 
-        optimizer = torch.optim.Optimizer(self.parameters(), dict())
+        optimizer = torch.optim.Optimizer(self.parameters(), {})
         params = optimizer.param_groups[0]["params"]
-
         flat_params = _get_flat_params(params)
         bounds = _get_flat_bounds(params)
-
         h_new = np.inf
+
         while (rho < rho_max) and (h_new > 0.25 * h or h_new == np.inf):
             # Magic
             sol = sopt.minimize(
@@ -403,11 +424,12 @@ class NotearsMLP(nn.Module, BaseEstimator):
                 jac=True,
                 bounds=bounds,
             )
-
             _update_params_from_flat(params, sol.x)
             h_new = self._h_func().item()
+
             if h_new > 0.25 * h:
                 rho *= 10
+
         alpha += rho * h_new
         return rho, alpha, h_new
 
@@ -420,7 +442,6 @@ class NotearsMLP(nn.Module, BaseEstimator):
             DAGness of the adjacency matrix
         """
         d = self.dims[0]
-        d_torch = torch.tensor(d).to(self.device)  # pylint: disable=not-callable
 
         # only consider the dag_layer for h(W) for compute efficiency
         dag_layer_weight = self.dag_layer_weight.view(d, -1, d)  # [j, m1, i]
@@ -430,18 +451,20 @@ class NotearsMLP(nn.Module, BaseEstimator):
 
         # modify the h(W) matrix to deal with expanded columns
         original_idxs = []
+
         for dist_type in self.dist_types:
             # modify the weight matrix to prevent spurious cycles with expended columns
             square_weight_mat = dist_type.modify_h(square_weight_mat)
             # gather the original idxs
             original_idxs.append(dist_type.idx)
+
         # original size is largest original index
         original_size = np.max(original_idxs) + 1
         # subselect the top LH corner of matrix which corresponds to original data
         square_weight_mat = square_weight_mat[:original_size, :original_size]
         # update d and d_torch to match the new matrix size
         d = square_weight_mat.shape[0]
-        d_torch = torch.tensor(d).to(self.device)  # pylint: disable=not-callable
+        d_torch = torch.tensor(d).to(self.device)
 
         # h = trace_expm(a) - d  # (Zheng et al. 2018)
         characteristic_poly_mat = (
@@ -472,14 +495,17 @@ class NotearsMLP(nn.Module, BaseEstimator):
             l2 regularisation term.
         """
         reg = 0.0
-        reg += torch.sum(self.dag_layer_weight ** 2)
+        reg += torch.sum(self.dag_layer_weight**2)
+
         for layer in self.loc_lin_layer_weights:
-            reg += torch.sum(layer.weight ** 2)
+            reg += torch.sum(layer.weight**2)
 
         # calculate the total number of elements used in the above sums
         n_elements = self.dag_layer_weight.numel()
+
         for layer in self.loc_lin_layer_weights:
             n_elements = n_elements + layer.weight.numel()
+
         return reg / n_elements * n_features
 
     def _calculate_adj(self, X: torch.Tensor, mean_effect: bool) -> torch.Tensor:
@@ -489,7 +515,6 @@ class NotearsMLP(nn.Module, BaseEstimator):
         For the linear case, this is just dag_layer_weight.
         For the nonlinear case, approximate the relationship using the gradient of X_hat wrt X.
         """
-
         # for the linear case, save compute by just returning the dag_layer weights
         if len(self.dims) <= 2:
             adj = (
@@ -500,12 +525,14 @@ class NotearsMLP(nn.Module, BaseEstimator):
             return adj
 
         _, n_features = X.shape
+
         # get the data X and reconstruction X_hat
         X = X.clone().requires_grad_()
         X_hat = self(X).sum(dim=0)  # shape = (n_features,)
 
-        adj = []
         # iterate over sums of reconstructed features
+        adj = []
+
         for j in range(n_features):
 
             # calculate the gradient of X_hat wrt X
@@ -517,6 +544,7 @@ class NotearsMLP(nn.Module, BaseEstimator):
             else:
                 # otherwise, use the average L1 of the gradient as the W
                 adj.append(torch.abs(ddx).mean(dim=0).unsqueeze(0))
+
         adj = torch.cat(adj, dim=0)
 
         # transpose to get the adjacency matrix
